/*******************************************************************************
* Title: LED_Graphics.h                                                                     
* Version: 1.0                                                                 
* Author: Jeff Nybo                                                            
* Date: April 23, 2014                                                                        
*                                                                              
* Description:                                                       
* This file contains various functions that are use to update and control the RGB pods,
* LED grid and LED rings.                                                                             
*******************************************************************************/

#ifndef LED_GRAPHICS_H
#define LED_GRAPHICS_H

/*************************************************
*               Global Variables                 *
*************************************************/
#define HT1632_UPDATE     FLAG1.b2
#define SCROLL_ACTIVE     FLAG1.b4
#define SCROLL_FINISHED   FLAG1.b5 

/*************************************************
*                   Constants                    *
*************************************************/
//Determines whether the pixels are inverted or not (Draw_Sine(x))
#define INVERTED          0
#define REGULAR           1

//The lower this is, the more sensitive the VU meter is. If you put it at max
//sensitivity you may pick up noise from the circuit itself. This may take a bit
//of trial and error to get things working but these values work great for me.
#define VU_SENSITIVTY     3

//This will adjust the offset of the VU meter. After adjusting VU_SENSITIVITY, 
//the VU meter can be tweaked with this value to cancel any unwanted noise that
//the ADC picks up.
#define VU_OFFSET		      20

//This determines how quick the bar will fall back down in the VU_Meter_Bar()
//function. The lower the value, the quicker it will fall.
#define FLOATING_BAR      7

//This is used to determine if a pods detection
//status has changed in Detect_Pod_Change(sensor_bits)
#define BITMASK_MASTER_SIDE       0x000003FF
#define BITMASK_SECONDARY_SIDE    0x0000FFC0

//Used to select the side for certain animations such as End_Blast()
#define MASTER_SIDE       1
#define SECONDARY_SIDE    2

/*************************************************
*                   Macros                       *
*************************************************/

/*************************************************
*              Function Prototypes               *
*************************************************/ 
void Fill_Grid(void);     
void Clear_Grid(void); 
void Invert_Grid(void);
void VU_Meter_Bar(void);   
void VU_Meter_Pod_Color(void);
void Draw_Border(UINT8 width); 
void VU_Meter_Circle_Out(void); 
void LED_Refresh_Grid(UINT16 *data);
void Set_Scrolling_Text(char text[64]);
void LED_Scoreboard(UINT32 pod_sensors);
void LED_Pixel(UINT8 px, UINT8 py, UINT8 state);
void Set_Text(UINT8 px, UINT8 py, char text[12]);   
void Draw_Circle(UINT8 px, UINT8 py, UINT8 radius);
void Draw_Rect(UINT8 px, UINT8 py, UINT8 sx, UINT8 sy);  

UINT8 Checkers(void);  
UINT8 Circle_Out(void);  
UINT8 Box_Grid_In(void);  
UINT8 Update_Text(void);  
UINT8 Corner_Circles(void);
UINT8 End_Blast(UINT8 side);
UINT8 Crossfade_Rings(void); 
UINT8 Load_Next_Letter(void);   
UINT8 Exploding_Circle(void); 
UINT8 Draw_Sine(UINT8 state);  
UINT8 Dual_Wave(UINT8 state);
        
UINT16 Read_VU(void);            

#endif
