/*******************************************************************************
* Title: PORT_Setup.c                                                          
* Version: 1.0                                                                 
* Author: Jeff Nybo                                                            
* Date: January 23, 2011                                                       
*                                                                              
* Description:                                                                 
* This file contains the functions that are used for setting up and            
* initializing the ports and register.                                         
*******************************************************************************/

#ifndef PORT_SETUP_C
#define PORT_SETUP_C

/*******************************************************************************
* Function: PORT_Init()                                                         
*                                                                              
* Variables:                                                                   
* N/A                                      
*
* Description:                                                                 
* This routine will initialize the ports as needed according to the schematic. 
* This should only be called once in a program.                                
*******************************************************************************/
void PORT_Init(void)
{
 CMCON  = 0x07;  //Disable Comparator Module
 ADCON1 = 0x0F;
 ADCON0 = 0x00;
 ADCON2 = 0xAB;

 PORTA = 0x00;
 PORTB = 0xFD; 
 PORTD = 0x00;
 PORTC = 0x00;
 PORTE = 0x02;

 TRISA = 0X01;  
 TRISB = 0X01;
 TRISC = 0X81;
 TRISD = 0X00;
 TRISE = 0X00;
}

/*******************************************************************************
* Function: Modules_Init(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function is used to initialize the modules/peripherals in the CPU.                                                                             
*******************************************************************************/
void Modules_Init(void)
{  
  //Initialize SPI module and enable it, High-Speed, Active to Idle, Idle Low 
  SPI_Init(SPI_ON | TX_A_I | IDLE_L | MAS_4);
}
#endif