/*******************************************************************************
* Title: RF Master Board                                                
* Version: 1.0                                                                 
* Author: Jeff Nybo                                                            
* Date: April 23, 2013                                       
*                                                                              
* Description:                                                                 
* This project contains the source code that is need to communicate through USB                                                                              
* with a PC. It is designed to be used with the BTP Control Center application that                                                                             
* I have made. Use this code at your own risk! I have not had much time to test
* it out and I wasn't even going to post it. I do not know much about USB and lots
* of the USB code is derived from Jan Axelson's code. Once you have this circuit
* set up and programmed, you can control the beer pong table wirelessly with it.                                                                              
*******************************************************************************/

/*************************************************
* Include device file and configuration settings *
*************************************************/
#include <p18f4550.h>
#include <stdio.h>
#include "StandardConfig.h"
#include "PORT_Setup.h"

//USB Driver files from Microchip.com
#include "usb.h"
#include "usb_function_hid.h"

#include "Delay_Setup.h"
#include "SPI_Setup.h"
#include "nRF24L01_Setup.h"
#include "RF_Controls.h"
#include "USB_Functions.h"

/*******************************************************************************
* Function: main()                                                             * 
*                                                                              *
* Variables:                                                                   *
* N/A                                                                          *
*                                                                              *
* Description:                                                                 *
* All of the code in the program is ran from inside this function. This        *
* function will never terminate as long as the MCU has power applied to it.    *
*******************************************************************************/
void main(void)
{ 
  //Initialize the I/O pins 
  PORT_Init();
  
  //Set up the required peripherals for this project (SPI only)
  Modules_Init();  
  
  //Initialize the nRF24L01+ as a transmitter 
  Tx_Initialize_RF_Module(); 
  
  //Initialize USB communication 
  InitializeSystem();
  
  while(1)
  {
    //Poll for USB data
    USBDeviceTasks();
    
    //Check to see if any USB data has been received
    ProcessIO();      
  }
}

/*************************************************
*                Included C Files                *
*************************************************/
#include "PORT_Setup.c"
#include "Delay_Setup.c"
#include "SPI_Setup.c"
#include "nRF24L01_Setup.c"
#include "RF_Controls.c"
#include "USB_Functions.c"
