/*******************************************************************************
* Title: StandardConfig.h                                                      *
* Version: 1.0                                                                 *
* Author: Jeff Nybo                                                            *
* Date: April 23, 2014                                                      *
*                                                                              *
* Description:                                                                 *
* This source file contains the configuration bits that will initialize the    *
* clock settings, code protection, etc. It also contains a few Quick Keywords  *
* to shorten the text length when declaring some variables.                    *
*******************************************************************************/

/*************************************************
*             Configuration Settings             *
*************************************************/
#pragma config PLLDIV   = 5         //20MHz crystal to 4MHz to 48MHz (PLL)
#pragma config CPUDIV   = OSC1_PLL2 //48MHz system clock
#pragma config USBDIV   = 2         // Clock source from 96MHz PLL/2
#pragma config FOSC     = HSPLL_HS	//Use PLL to achieve 48MHz
#pragma config FCMEN    = OFF				//Fail safe clock monitor
#pragma config IESO     = OFF				//Oscillator switchover mode disabled
#pragma config PWRT     = OFF				//Power Up Timer Disabled
#pragma config BOR      = OFF				//Brown out reset disabled
#pragma config BORV     = 3					//Minimum Brown Out Voltage 
#pragma config VREGEN   = ON        //USB Voltage Regulator
#pragma config WDT      = OFF				//Watchdog Timer Disabled
#pragma config WDTPS    = 32768			//Watchdog Postscaler
#pragma config MCLRE    = ON				//MCLR Pin Enabled
#pragma config LPT1OSC  = OFF				//Low Power Timer1 Oscillator Disabled
#pragma config PBADEN   = OFF				//PORTB<4:0> configured as digital on reset
#pragma config CCP2MX   = OFF				//CCP2 is multiplexed with RB3
#pragma config STVREN   = ON				//Stack Full Underflow will cause reset
#pragma config LVP      = OFF				//Low Voltage Progrtamming Disabled
#pragma config ICPRT    = OFF       //Dedicated In-Circuit Debug/Programming
#pragma config XINST    = OFF       //Extended Instruction Set
#pragma config CP0      = OFF				//All types of code protection is disabled
#pragma config CP1      = OFF
#pragma config CP2      = OFF
#pragma config CP3      = OFF
#pragma config CPB      = OFF
#pragma config CPD      = OFF
#pragma config WRT0     = OFF
#pragma config WRT1     = OFF
#pragma config WRT2     = OFF
#pragma config WRT3     = OFF
#pragma config WRTB     = OFF       // Boot Block Write Protection
#pragma config WRTC     = OFF
#pragma config WRTD     = OFF
#pragma config EBTR0    = OFF
#pragma config EBTR1    = OFF
#pragma config EBTR2    = OFF
#pragma config EBTR3    = OFF
#pragma config EBTRB    = OFF 

/*************************************************
*                   EEPROM                       *
*************************************************/

/*************************************************
*                 Quick Keywords                 *
*************************************************/
typedef unsigned long int uint32;
typedef signed long int int32;

typedef unsigned short long uint24;
typedef signed short long int24;

typedef unsigned int uint16;
typedef signed int int16;

typedef unsigned char uint8;
typedef signed char int8;

typedef unsigned long int DWORD;
typedef unsigned long int UINT32;
typedef signed long int INT32;

typedef unsigned short long UINT24;
typedef signed short long INT24;

typedef unsigned int UINT16;
typedef unsigned int WORD;
typedef signed int INT16;

typedef unsigned char UINT8;
typedef unsigned char BYTE;
typedef signed char INT8;

typedef const far rom char* romstrp;

/*************************************************
*                  Constants                     *
*************************************************/ 
#define FOSC  48000000L
#define FCYC  (FOSC / 4)

#define TOSC  (1 / FOSC)
#define TCYC  (4 / FOSC)

#define HIGH  0x01
#define LOW   0x00

//#define TRUE  0x01
//#define FALSE 0x00

#define ON    0x01
#define OFF   0x00

/*************************************************
*                   Macros                       *
*************************************************/
#define BitSet(var,bitno) (var|=(1<<bitno))
#define BitClr(var,bitno) (var&=~(1<<bitno))
#define BitTst(var,bitno) (var&(1<<bitno))

/*************************************************
*               Global Variables                 *
*************************************************/
#pragma udata
    #pragma udata USB_VARIABLES=0x500

  unsigned char USB_Rx[64];
  unsigned char USB_Tx[64];
#pragma udata
