/*******************************************************************************
* Title: USB_Functions.c                                                       
* Version: 1.0                                                                 
* Author: Jeff Nybo (Derived from Jan Axelson's Source Code)                                                            
* Date: April 23, 2014                                                                       
*                                                                              
* Description:                                                                 
* This file contains functions that are needed in order to properly communicate
* over the USB protocol.    
*******************************************************************************/

/*******************************************************************************
* Function: InitializeSystem(void)                                             
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will initialize the USB module on the PIC and allow us to communicate                                                                              
* over the USB protocol.                                                                             
*******************************************************************************/
void InitializeSystem(void)
{
  //Initialize USB modules (usb_device.h)
  UserInit(); 
  USBDeviceInit();
}

/*******************************************************************************
* Function: ProcessIO(void)                                                                   
*                                                                              
* Variables:                                                                   
* N/A                                                                             
*                                                                              
* Description:                                                                 
* This function will check to see if there is any USB data that is ready to be                                                                               
* received from the host.                                                                             
*******************************************************************************/
void ProcessIO(void)
{ 
    // Check for USB tasks
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;
    
    if(!HIDRxHandleBusy(USBOutHandle))				//Check if data was received from the host.
    { 
      //Check what command has been sent
      Tx_Check_CMD();
       
      //Re-arm the OUT endpoint for the next packet
      USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&USB_Rx,64);
    }
}

/*******************************************************************************
* Function:                                                                    
*                                                                              
* Variables:                                                                   
*                                                                              
*                                                                              
* Description:                                                                 
*                                                                               
*                                                                              
*******************************************************************************/
void USBCBCheckOtherReq(void)
{
    USBCheckHIDRequest();
}

/*******************************************************************************
* Function:                                                                   
*                                                                             
* Variables:                                                                  
*                                                                             
*                                                                             
* Description:                                                                
*                                                                              
*                                                                             
*******************************************************************************/
void USBCBErrorHandler(void)
{

}

/*******************************************************************************
* Function:                                                                    
*                                                                              
* Variables:                                                                   
*                                                                              
*                                                                              
* Description:                                                                 
*                                                                               
*                                                                              
*******************************************************************************/
void USBCBInitEP(void)
{
    //enable the HID endpoint
    USBEnableEndpoint(HID_EP,USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
    //Re-arm the OUT endpoint for the next packet
    USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&USB_Rx,64);
}

/*******************************************************************************
* Function:                                                                    
*                                                                              
* Variables:                                                                   
*                                                                              
*                                                                              
* Description:                                                                 
*                                                                               
*                                                                              
*******************************************************************************/
void USBCBSendResume(void)
{
    static WORD delay_count;
    
    USBResumeControl = 1;          
    
    delay_count = 1800U;                
    do
    {
        delay_count--;
    }while(delay_count);
    USBResumeControl = 0;
}

/*******************************************************************************
* Function:                                                                    
*                                                                              
* Variables:                                                                   
*                                                                              
*                                                                              
* Description:                                                                 
*                                                                               
*                                                                              
*******************************************************************************/
void USBCB_SOF_Handler(void)
{

}

/*******************************************************************************
* Function:                                                                    
*                                                                              
* Variables:                                                                   
*                                                                              
*                                                                              
* Description:                                                                 
*                                                                               
*                                                                              
*******************************************************************************/
void USBCBStdSetDscHandler(void)
{

}

/*******************************************************************************
* Function:                                                                    
*                                                                              
* Variables:                                                                   
*                                                                              
*                                                                              
* Description:                                                                 
*                                                                               
*                                                                              
*******************************************************************************/
void USBCBSuspend(void)
{

}

/*******************************************************************************
* Function:                                                                    
*                                                                              
* Variables:                                                                   
*                                                                              
*                                                                              
* Description:                                                                 
*                                                                               
*                                                                              
*******************************************************************************/
void USBCBWakeFromSuspend(void)
{

}

/*******************************************************************************
* Function:                                                                    
*                                                                              
* Variables:                                                                   
*                                                                              
*                                                                              
* Description:                                                                 
*                                                                               
*                                                                              
*******************************************************************************/
void UserInit(void)
{
    USBOutHandle = 0;
    USBInHandle = 0;
}

/*******************************************************************************
* Function:                                                                    
*                                                                              
* Variables:                                                                   
*                                                                              
*                                                                              
* Description:                                                                 
*                                                                               
*                                                                              
*******************************************************************************/
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
    switch(event)
    {
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        case EVENT_TRANSFER:
            Nop();
            break;
        default:
            break;
    }      
    return TRUE; 
}
