/*******************************************************************************
* Title: nRF24L01_Setup.h                                                      
* Version: 1.0                                                                 
* Author: Jeff Nybo                                                            
* Date: April 23, 2014                                                        
*                                                                              
* Description:                                                                 
* This file contains the functions and commands that are used to set up and control                                                                             
* the operation of nRF24L01 RF Modules.                                                                                                            
*******************************************************************************/

#ifndef NRF24L01_SETUP_H
#define NRF24L01_SETUP_H

/*************************************************
*                  Constants                     *
*************************************************/ 
#define RF_ADDR_WIDTH           0x05
#define RF_RX_MODE              0x01
#define RF_RX_ACTIVE            0x01
#define RF_RX_STANDBY           0x00
#define RF_TX_MODE              0x00
#define RF_TX_ACTIVE            0x00

/*************************************************
*                   Commands                     *
*************************************************/ 
#define RF_READ_REG             0x00  
#define RF_WRITE_REG            0x20  
#define RF_RD_RX_PLOAD          0x61  
#define RF_WR_TX_PLOAD          0xA0  
#define RF_WR_TX_NOACK          0xB0  
#define RF_FLUSH_TX             0xE1  
#define RF_FLUSH_RX             0xE2  
#define RF_REUSE_TX_PL          0xE3  
#define RF_NOP                  0xFF  

/*************************************************
*                  Registers                     *
*************************************************/ 
#define RF_CONFIG               0x00  
#define RF_EN_AA                0x01  
#define RF_EN_RXADDR            0x02  
#define RF_SETUP_AW             0x03  
#define RF_SETUP_RETR           0x04  
#define RF_CH                   0x05  
#define RF_SETUP                0x06  
#define RF_STATUS               0x07  
#define RF_OBSERVE_TX           0x08  
#define RF_RPD                  0x09  
#define RF_RX_ADDR_P0           0x0A  
#define RF_RX_ADDR_P1           0x0B  
#define RF_RX_ADDR_P2           0x0C  
#define RF_RX_ADDR_P3           0x0D  
#define RF_RX_ADDR_P4           0x0E  
#define RF_RX_ADDR_P5           0x0F  
#define RF_TX_ADDR              0x10  
#define RF_RX_PW_P0             0x11  
#define RF_RX_PW_P1             0x12  
#define RF_RX_PW_P2             0x13  
#define RF_RX_PW_P3             0x14  
#define RF_RX_PW_P4             0x15  
#define RF_RX_PW_P5             0x16  
#define RF_FIFO_STATUS          0x17  

/*************************************************
*            Default Register Values             *
*************************************************/ 
#define RF_CONFIG_DV			      0x08
#define RF_EN_AA_DV			        0x3F
#define RF_EN_RXADDR_DV		      0x03
#define RF_SETUP_AW_DV		      0x03
#define RF_SETUP_RETR_DV		    0x03
#define RF_RF_CH_DV			        0x02
#define RF_RF_SETUP_DV		      0x0F
#define RF_STATUS_DV			      0x0E
#define RF_OBSERVE_TX_DV		    0x00
#define RF_CD_DV				        0x00
#define RF_RX_ADDR_P0_B0_DV	    0xE7
#define RF_RX_ADDR_P0_B1_DV	    0xE7
#define RF_RX_ADDR_P0_B2_DV	    0xE7
#define RF_RX_ADDR_P0_B3_DV	    0xE7
#define RF_RX_ADDR_P0_B4_DV	    0xE7
#define RF_RX_ADDR_P1_B0_DV	    0xC2
#define RF_RX_ADDR_P1_B1_DV	    0xC2
#define RF_RX_ADDR_P1_B2_DV	    0xC2
#define RF_RX_ADDR_P1_B3_DV	    0xC2
#define RF_RX_ADDR_P1_B4_DV  	  0xC2
#define RF_RX_ADDR_P2_DV		    0xC3
#define RF_RX_ADDR_P3_DV		    0xC4
#define RF_RX_ADDR_P4_DV		    0xC5
#define RF_RX_ADDR_P5_DV		    0xC6
#define RF_TX_ADDR_B0_DV		    0xE7
#define RF_TX_ADDR_B1_DV		    0xE7
#define RF_TX_ADDR_B2_DV	    	0xE7
#define RF_TX_ADDR_B3_DV	    	0xE7
#define RF_TX_ADDR_B4_DV	    	0xE7
#define RF_RX_PW_P0_DV	      	0x00
#define RF_RX_PW_P1_DV	      	0x00
#define RF_RX_PW_P2_DV	      	0x00
#define RF_RX_PW_P3_DV	      	0x00
#define RF_RX_PW_P4_DV	      	0x00
#define RF_RX_PW_P5_DV	      	0x00
#define RF_FIFO_STATUS_DV    	  0x11


/*************************************************
*          Register Bitwise Definitions          *
*************************************************/ 
//CONFIG bits
#define RF_CONFIG_ALL_IRQS      0x70
#define	RF_CONFIG_MASK_RX_DR	  0x40
#define	RF_CONFIG_MASK_TX_DS	  0x20
#define	RF_CONFIG_MASK_MAX_RT	  0x10
#define	RF_CONFIG_EN_CRC		    0x08
#define	RF_CONFIG_CRCO		      0x04
#define	RF_CONFIG_PWR_UP		    0x02
#define	RF_CONFIG_PRIM_RX		    0x01

//EN_AA bits
#define RF_EN_AA_ALL		        0x3F
#define RF_EN_AA_P5		          0x20
#define RF_EN_AA_P4		          0x10
#define RF_EN_AA_P3		          0x08
#define RF_EN_AA_P2		          0x04
#define RF_EN_AA_P1		          0x02
#define RF_EN_AA_P0		          0x01
#define RF_EN_AA_NONE	          0x00

//EN_RXADDR bits
#define RF_EN_RXADDR_ALL			  0x3F
#define RF_EN_RXADDR_P5	  		  0x20
#define RF_EN_RXADDR_P4	  		  0x10
#define RF_EN_RXADDR_P3	  		  0x08
#define RF_EN_RXADDR_P2	  		  0x04
#define RF_EN_RXADDR_P1	  		  0x02
#define RF_EN_RXADDR_P0	  		  0x01
#define RF_EN_RXADDR_NONE			  0x00

//SETUP_AW bits
#define RF_SETUP_AW_5B	        0x03
#define RF_SETUP_AW_4B	        0x02
#define RF_SETUP_AW_3B	        0x01

//SETUP_RETR bits
#define RF_SETUP_RETR_ARD		  	0xF0
#define RF_SETUP_RETR_ARD_4000	0xF0
#define RF_SETUP_RETR_ARD_3750	0xE0
#define RF_SETUP_RETR_ARD_3500	0xD0
#define RF_SETUP_RETR_ARD_3250	0xC0
#define RF_SETUP_RETR_ARD_3000	0xB0
#define RF_SETUP_RETR_ARD_2750	0xA0
#define RF_SETUP_RETR_ARD_2500	0x90
#define RF_SETUP_RETR_ARD_2250	0x80
#define RF_SETUP_RETR_ARD_2000	0x70
#define RF_SETUP_RETR_ARD_1750	0x60
#define RF_SETUP_RETR_ARD_1500	0x50
#define RF_SETUP_RETR_ARD_1250	0x40
#define RF_SETUP_RETR_ARD_1000	0x30
#define RF_SETUP_RETR_ARD_750		0x20
#define RF_SETUP_RETR_ARD_500		0x10
#define RF_SETUP_RETR_ARD_250		0x00
#define RF_SETUP_RETR_ARC			  0x0F
#define RF_SETUP_RETR_ARC_15		0x0F
#define RF_SETUP_RETR_ARC_14		0x0E
#define RF_SETUP_RETR_ARC_13		0x0D
#define RF_SETUP_RETR_ARC_12		0x0C
#define RF_SETUP_RETR_ARC_11		0x0B
#define RF_SETUP_RETR_ARC_10		0x0A
#define RF_SETUP_RETR_ARC_9		  0x09
#define RF_SETUP_RETR_ARC_8		  0x08
#define RF_SETUP_RETR_ARC_7		  0x07
#define RF_SETUP_RETR_ARC_6		  0x06
#define RF_SETUP_RETR_ARC_5		  0x05
#define RF_SETUP_RETR_ARC_4		  0x04
#define RF_SETUP_RETR_ARC_3		  0x03
#define RF_SETUP_RETR_ARC_2		  0x02
#define RF_SETUP_RETR_ARC_1		  0x01
#define RF_SETUP_RETR_ARC_0		  0x00

//RF_SETUP bits
#define RF_SETUP_PLL_LOCK	      0x10
#define RF_SETUP_RF_DR		      0x08
#define RF_SETUP_RF_PWR	        0x06
#define RF_SETUP_RF_PWR_0	      0x06
#define RF_SETUP_RF_PWR_6 	    0x04
#define RF_SETUP_RF_PWR_12	    0x02
#define RF_SETUP_RF_PWR_18	    0x00

//STATUS bits
#define RF_STATUS_ALL_IRQ				0x70
#define RF_STATUS_RX_DR					0x40
#define RF_STATUS_TX_DS					0x20
#define RF_STATUS_MAX_RT				0x10
#define RF_STATUS_RX_PIPES  		0x0E
#define RF_STATUS_RX_P_5			  0x0A
#define RF_STATUS_RX_P_4		  	0x08
#define RF_STATUS_RX_P_3			  0x06
#define RF_STATUS_RX_P_2			  0x04
#define RF_STATUS_RX_P_1			  0x02
#define RF_STATUS_RX_P_0			  0x00
#define RF_STATUS_TX_FULL				0x01

//OBSERVE_TX bits
#define RF_OBSERVE_TX_PLOS_CNT	0xF0
#define RF_OBSERVE_TX_ARC_CNT		0x0F

//RPD bits
#define RF_RPD_RPD  			      0x01

//FIFO_STATUS bits
#define RF_FIFO_STATUS_TX_REUSE	  0x40
#define RF_FIFO_STATUS_TX_FULL	  0x20
#define RF_FIFO_STATUS_TX_EMPTY	  0x10
#define RF_FIFO_STATUS_RX_FULL	  0x02
#define RF_FIFO_STATUS_RX_EMPTY	  0x01


/*************************************************
*               Pin Declarations                 *
*************************************************/ 
#define SPI_SDO        _LATG8
#define SPI_SDI        _LATG7
#define SPI_SCK        _LATG6
          
#define RF_IRQ         PORTBbits.RB11
#define RF_CE          _LATB8
#define RF_CSN         _LATB9

//I am using SPI2 ro send data
#define SPI_Send(x)		SPI2_Send(x)

/*************************************************
*                   Macros                       *
*************************************************/
#define RX_STANDBY()    (RF_CE = 0)
#define RX_ACTIVE()     (RF_CE = 1)

/*************************************************
*              Function Prototypes               *
*************************************************/ 
void nRF24L01_Init(UINT8 config, 
                   UINT8 rf_mode, 
                   UINT8 en_aa, 
                   UINT8 en_rxaddr, 
                   UINT8 setup_aw, 
                   UINT8 setup_retr,
                   UINT8 channel,
                   UINT8 setup);

void nRF24L01_SetRxAddress(UINT8 *rx_addr_p0, 
                           UINT8 *rx_addr_p1, 
                           UINT8 addr_width, 
                           UINT8 *rx_pw);  

UINT8 nRF24L01_ARC_CNT(void);
UINT8 nRF24L01_PLOS_CNT(void);             
UINT8 nRF24L01_Check_ACK(void); 
UINT8 nRF24L01_SignalStrength(void);
UINT8 nRF24L01_Get_RX_FIFO_Pipe(void);
UINT8 nRF24L01_Rx(UINT8 *data, UINT8 len);
UINT8 nRF24L01_Tx_SB(UINT8 *data, UINT8 len);
UINT8 nRF24L01_Check_Ping_SB(UINT8 *data, UINT8 len);
UINT8 nRF24L01_Ping_Pipe(UINT8 pipe, UINT8 *data, UINT8 len);
UINT8 nRF24L01_Check_Ping(UINT8 pipe, UINT8 *data, UINT8 len);
UINT8 nRF24L01_Ping_Pipe_SB(UINT8 pipe, UINT8 *data, UINT8 len);

void nRF24L01_CMD(UINT8 name);
void nRF24L01_Transmit(void);
void nRF24L01_Clear_All(void);
void nRF24L01_PowerDown(void); 
void nRF24L01_Clear_PLOS(void);
void nRF24L01_ReTransmit(void);
void nRF24L01_Flush_FIFOs(void);
void nRF24L01_Clear_IRQ(UINT8 irq);
void nRF24L01_PowerUp(UINT8 active);
void nRF24L01_Disable_Pipe(UINT8 pipe);
void nRF24L01_Tx(UINT8 *data, UINT8 len);
void nRF24L01_Enable_IRQ_Pin(UINT8 irq);
void nRF24L01_Disable_IRQ_Pin(UINT8 irq);
void nRF24L01_Mode(UINT8 mode, UINT8 active);
void nRF24L01_Set_RF_Channel(UINT8 channel);
void nRF24L01_Enable_Pipe(UINT8 pipe, UINT8 aa_mode);
void nRF24L01_ReadReg(UINT8 reg, UINT8 *data, UINT8 len);
void nRF24L01_WriteReg(UINT8 reg, UINT8 *data, UINT8 len);
void nRF24L01_SetTxAddress(UINT8 *tx_addr, UINT8 addr_width);
void nRF24L01_Set_Tx_ACK_Address(UINT8 *addr, UINT8 addr_width);
void nRF24L01_Set_Pipe0_Address(UINT8 *rx_addr_p0, UINT8 addr_width);

#endif
